﻿using System;
using System.Data.OleDb;

using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 연결
        /// </summary>
        private OleDbConnection connection;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.connection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=nwind.mdb");

            this.gridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            this.gridView.OptionsBehavior.AutoPopulateColumns = true;

            this.gridControl.DataSource = Product.GetProductList(this.connection);

            this.gridView.BestFitColumns();

            this.gridView.TopRowChanged += gridView_TopRowChanged;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그리드 뷰 상위 행 변경시 처리하기 - gridView_TopRowChanged(sender, e)

        /// <summary>
        /// 그리드 뷰 상위 행 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_TopRowChanged(object sender, EventArgs e)
        {
            UpdateStatus();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 마지막 표시 행 핸들 구하기 - GetLastVisibleRowHandle()

        /// <summary>
        /// 마지막 표시 행 핸들 구하기
        /// </summary>
        /// <returns>마지막 표시 행 핸들</returns>
        private int GetLastVisibleRowHandle()
        {
            int rowHandle = 0;

            for(int visibleIndex = this.gridView.TopRowIndex + 1; visibleIndex < this.gridView.RowCount; visibleIndex++)
            {
                rowHandle = this.gridView.GetVisibleRowHandle(visibleIndex);

                if(this.gridView.IsRowVisible(rowHandle) == RowVisibleState.Hidden)
                {
                    break;
                }
            }

            return rowHandle - 1;
        }

        #endregion
        #region 상태 업데이트하기 - UpdateStatus()

        /// <summary>
        /// 상태 업데이트하기
        /// </summary>
        private void UpdateStatus()
        {
            int topRowHandle = this.gridView.GetVisibleRowHandle(this.gridView.TopRowIndex);

            double topRowValue = Convert.ToDouble
            (
                this.gridView.GetRowCellValue
                (
                    topRowHandle,
                    this.gridView.Columns["UnitPrice"]
                )
            );

            double bottomRowValue = Convert.ToDouble
            (
                this.gridView.GetRowCellValue
                (
                    GetLastVisibleRowHandle(),
                    this.gridView.Columns["UnitPrice"]
                )
            );

            this.statusLabelControl.Text = string.Format("단가 : {0:c2} - {1:c2}", topRowValue, bottomRowValue);
        }

        #endregion
    }
}